@@ -3,6 +3,8 @@ from __future__ import unicode_literals
 
 import datetime
 import itertools
+import os
+import platform
 import time
 from decimal import Decimal
 
@@ -16,14 +18,23 @@ from django.test import TransactionTestCase
 from django.test.utils import override_settings
 from django.utils import timezone
 from django.utils.six import text_type
+from django.utils.unittest import skipIf, skipUnless
 from httpretty import httprettified, HTTPretty
 from mock import Mock, NonCallableMock, NonCallableMagicMock, patch, MagicMock
 
 from filemaker import fields, FileMakerValidationError, FileMakerModel
 from filemaker.base import deep_getattr
+from filemaker.exceptions import FileMakerServerError
 from filemaker.manager import RawManager, Manager
+from filemaker.parser import FMXMLObject, FMDocument
 from filemaker.utils import get_field_class
 
+try:
+    from django.utils.encoding import force_bytes
+except ImportError:
+    # Django 1.4.x
+    from django.utils.encoding import smart_bytes as force_bytes  # NOQA
+
 
 class TestFilemakerFields(TransactionTestCase):
 
@@ -269,6 +280,10 @@ class TestFilemakerFields(TransactionTestCase):
         self.assertEqual(d.second, 2)
         self.assertTrue(isinstance(d, datetime.datetime))
 
+    @skipIf(
+        platform.python_implementation().lower() == 'pypy',
+        'PyPy won\'t throw the expected OverflowError in this case.'
+    )
     @override_settings(USE_TZ=False)
     def test_datetime_parse_overflow(self):
         '''
@@ -1378,3 +1393,207 @@ class TestUtils(TransactionTestCase):
             get_field_class('django.db.models.fields.IntegerField'),
             fields.IntegerField
         )
+
+
+@skipUnless(
+    os.path.exists(os.path.join(
+        os.path.dirname(__file__),
+        '../test_xml/test_xml.xml'
+    )),
+    'Test XML file test_xml.xml not found.'
+)
+class TestParser(TransactionTestCase):
+
+    def setUp(self):
+        with open(os.path.join(
+                os.path.dirname(__file__), '../test_xml/test_xml.xml')) as f:
+            self.xml = force_bytes(f.read())
+        self.fm_object = FMXMLObject(self.xml)
+
+    def test_parser_data(self):
+        self.assertEqual(self.fm_object.data, self.xml)
+
+    def test_parser_database(self):
+        self.assertEqual(
+            self.fm_object.database,
+            {
+                'database': 'art',
+                'date-format': 'MM/dd/yyyy',
+                'layout': 'web3',
+                'table': 'art',
+                'time-format': 'HH:mm:ss',
+                'timestamp-format': 'MM/dd/yyyy HH:mm:ss',
+                'total-count': '12',
+            }
+        )
+
+    def test_parser_errorcode(self):
+        self.assertEqual(self.fm_object.errorcode, 0)
+
+    def test_parser_field_names(self):
+        self.assertEqual(
+            self.fm_object.field_names,
+            ['Title', 'Artist', 'Style', 'length']
+        )
+
+    def test_parser_metadata(self):
+        self.assertEqual(
+            self.fm_object.metadata,
+            {
+                'Artist': {
+                    'auto-enter': 'no',
+                    'four-digit-year': 'no',
+                    'global': 'no',
+                    'max-repeat': '1',
+                    'name': 'Artist',
+                    'not-empty': 'no',
+                    'numeric-only': 'no',
+                    'result': 'text',
+                    'time-of-day': 'no',
+                    'type': 'normal'
+                },
+                'Style': {
+                    'auto-enter': 'no',
+                    'four-digit-year': 'no',
+                    'global': 'no',
+                    'max-repeat': '1',
+                    'name': 'Style',
+                    'not-empty': 'no',
+                    'numeric-only': 'no',
+                    'result': 'text',
+                    'time-of-day': 'no',
+                    'type': 'normal'
+                },
+                'Title': {
+                    'auto-enter': 'no',
+                    'four-digit-year': 'no',
+                    'global': 'no',
+                    'max-repeat': '1',
+                    'name': 'Title',
+                    'not-empty': 'no',
+                    'numeric-only': 'no',
+                    'result': 'text',
+                    'time-of-day': 'no',
+                    'type': 'normal'
+                },
+                'length': {
+                    'auto-enter': 'no',
+                    'four-digit-year': 'no',
+                    'global': 'no',
+                    'max-repeat': '1',
+                    'name': 'length',
+                    'not-empty': 'no',
+                    'numeric-only': 'no',
+                    'result': 'number',
+                    'time-of-day': 'no',
+                    'type': 'calculation'
+                }
+            }
+        )
+
+    def test_parser_product(self):
+        self.assertEqual(
+            self.fm_object.product,
+            {
+                'build': '12/31/2012',
+                'name': 'FileMaker Web Publishing Engine',
+                'version': '0.0.0.0',
+            }
+        )
+
+    def test_parser_resultset(self):
+        self.assertEqual(
+            self.fm_object.resultset,
+            [
+                {
+                    'Artist': 'Claude Monet',
+                    'MODID': 6,
+                    'RECORDID': 14,
+                    'Style': '',
+                    'Title': 'Spring in Giverny 3',
+                    'length': '19'
+                }
+            ]
+        )
+
+    def test_resultset_len(self):
+        self.assertEqual(len(self.fm_object), 1)
+
+    def test_resultset_getitem(self):
+        self.assertEqual(self.fm_object[0], self.fm_object.resultset[0])
+
+    def test_fm_document_get_attrs(self):
+        self.assertEqual(
+            self.fm_object.resultset[0].length,
+            self.fm_object.resultset[0]['length']
+        )
+
+    def test_fm_document_set_attrs(self):
+        doc = FMDocument(a=1, b=2)
+        doc.a = 3
+        self.assertEqual(doc['a'], 3)
+        self.assertEqual(doc.a, 3)
+        doc['a'] = 4
+        self.assertEqual(doc['a'], 4)
+        self.assertEqual(doc.a, 4)
+
+    @skipUnless(
+        os.path.exists(os.path.join(
+            os.path.dirname(__file__),
+            '../test_xml/test_error_xml.xml')
+        ),
+        'Test XML file test_error_xml.xml not found.'
+    )
+    def test_errorcode(self):
+        with open(os.path.join(
+                os.path.dirname(__file__),
+                '../test_xml/test_error_xml.xml')) as f:
+            xml = force_bytes(f.read())
+        with self.assertRaises(FileMakerServerError):
+            FMXMLObject(xml)
+
+    @skipUnless(
+        os.path.exists(os.path.join(
+            os.path.dirname(__file__),
+            '../test_xml/test_broken_xml.xml')
+        ),
+        'Test XML file test_broken_xml.xml not found.'
+    )
+    def test_broken_xml(self):
+        with open(os.path.join(
+                os.path.dirname(__file__),
+                '../test_xml/test_broken_xml.xml')) as f:
+            xml = force_bytes(f.read())
+        with self.assertRaises(FileMakerServerError):
+            FMXMLObject(xml)
+
+    @skipUnless(
+        os.path.exists(os.path.join(
+            os.path.dirname(__file__),
+            '../test_xml/test_invalid_xml.xml')
+        ),
+        'Test XML file test_invalid_xml.xml not found.'
+    )
+    def test_invalid_xml(self):
+        with open(os.path.join(
+                os.path.dirname(__file__),
+                '../test_xml/test_invalid_xml.xml')) as f:
+            xml = force_bytes(f.read())
+        with self.assertRaises(FileMakerServerError):
+            FMXMLObject(xml)
+
+    @skipUnless(
+        os.path.exists(os.path.join(
+            os.path.dirname(__file__),
+            '../test_xml/test_empty_xml.xml')
+        ),
+        'Test XML file test_empty_xml.xml not found.'
+    )
+    def test_empty_xml(self):
+        with open(os.path.join(
+                os.path.dirname(__file__),
+                '../test_xml/test_empty_xml.xml')) as f:
+            xml = force_bytes(f.read())
+        fm_object = FMXMLObject(xml)
+        self.assertEqual(fm_object.resultset, [])
+        self.assertEqual(len(fm_object), 0)
